package mc600.boardgames;

import java.util.Vector;

public class TanboBoard {
	protected int dimension;
	protected int[][] positions;
	protected boolean gameIsOver;
	protected int victory;
	protected int nMoves;
	protected Vector blackRoots;
	protected Vector whiteRoots;

	public static final int NOBODY = -1;
	public static final int FREE = 0;
	public static final int BLACK = 1;
	public static final int WHITE = 2;

	public TanboBoard(int dim) throws Exception {

		// seta dimensoes do tabuleiro 
		setDimension(dim);

		// cria posicoes, que sao a composicao basica do tabuleiro
		positions = new int[getDimension()][getDimension()];

		for (int i = 0; i < getDimension(); i++)
			for (int j = 0; j < getDimension(); j++)
				positions[i][j] = FREE;

		// inicializa condicoes de jogo
		gameIsOver = false;
		victory = 0;
		nMoves = 0;
		blackRoots = new Vector();
		whiteRoots = new Vector();
		initializeRoots();
	}

  /*
   *  Retorna um vetor de raizes contendo as raizes do jogador preto.
   */
	public Root[] getBlackRoots() {
		return (Root[]) blackRoots.toArray(new Root[0]);
	}

	/*
	 *  Retorna um vetor de raizes contendo as raizes do jogador branco.
	 */
	public Root[] getWhiteRoots() {
		return (Root[]) whiteRoots.toArray(new Root[0]);
	}

  /*
   * Retorna um vetor bidimensional de inteiros representando os estado atual 
   * do tabuleiro.  A posicao A1 corresponde `a posicao (0,0) do vetor, B1 a (1,0),
   * etc.
   */
	public int[][] getBoardPositions() {
		return positions;
	}

  /*
   * Retorna a dimensao do tabuleiro.
   */
	public int getDimension() {
		return dimension;
	}

  /*
   * Retorna TRUE se e somente se o jogo terminou.
   */
	public boolean gameOver() {
		return gameIsOver;
	}

  /*
   *  Retorna TRUE se e somente se o jogador preto ganhou
   * ( = branco nao tem mais raizes.)
   */
	public boolean victoryBlack() {
		return victory == BLACK;
	}

	/*
	 *  Retorna TRUE se e somente se o jogador branco ganhou
	 * ( = preto nao tem mais raizes.)
	 */
	public boolean victoryWhite() {
		return victory == WHITE;
	}

	/**
	 * Retorna true caso o jogador tenha direito a mais um lance.
	 * (no caso de Connect Four, sempre retorna false).
	 *
	 * O tabuleiro e' formado por uma array bidimensional contendo
	 * 0, 1 ou 2, para indicar casa vazia, ocupada por P1 ou ocupada
	 * por P2, respectivamente.
	 *
	 * Depois de cada lance e' feita a verificacao para saber
	 * se o jogo acabou, ou seja, se o jogador fechou uma linha
	 * com pelo menos 4 casas.
	 *
	 * Creation date: (3/24/00 3:27:06 AM)
	 * @return boolean
	 * @param move java.lang.String
	 */
	public boolean play(String move, boolean isBlack)
		throws InvalidMoveException {

		BoardPosition position = new BoardPosition(move, dimension);

		//verifica se cordenadas estao dentro do tabuleiro
		if (!position.inBoard())
			throw new InvalidMoveException(
				"Lance fora do tabuleiro <" + move + "> ");

		int turn = isBlack ? BLACK : WHITE;

		//verifica se casa esta' vazia e a de baixo ocupada (caso exista)
		if (!isValid(position, turn))
			throw new InvalidMoveException("Lance invalido <" + move + "> ");

		// marca lance
		mark(position, turn);
		nMoves++;

		// ve^ se fechou
		int winner = winner();
		if (winner != NOBODY) {
			gameIsOver = true;
			victory = winner;
		}

		return false;
	}

	// Estabelece a dimensao do tabuleiro
	protected void setDimension(int newDim) throws Exception {
		// Tanbo s pode ser jogado em tabuleiro 9X9 ou 19X19
		if ((newDim != 9) && (newDim != 19))
			throw new Exception("Dimensao do tabuleiro invalida.\n");
		dimension = newDim;
	}

  /*
   * Retorna o valor de uma posicao no vetor que representa o tabuleiro.
   * Este valor equivale a  um dos atributos estaticos: BLACK, WHITE e
   * FREE desta mesma classe. 
   */
	public int conteudo(BoardPosition casa) {
		return positions[casa.getX()][casa.getY()];
	}

    /*
     * Retorna TRUE se a posicao no tabuleiro ainda estiver vazia.
     */
	public boolean empty(BoardPosition casa) {
		return conteudo(casa) == FREE;
	}

    /*
     *  Retorna TRUE se a posicao no tabuleiro esta ocupada.
     */
	public boolean occupied(BoardPosition casa) {
		return !empty(casa);
	}

    /*
     *  Efetiva a jogada: marca a posicao, reorganiza raizes, etc...
     */
	public void mark(BoardPosition position, int turn)
		throws InvalidMoveException {
		boolean accepted = false;
		boolean removed = false;
		Vector playerRoots = blackRoots;
		Vector adversaryRoots = whiteRoots;
		Root extended = null;
		if (turn == WHITE) {
			playerRoots = whiteRoots;
			adversaryRoots = blackRoots;
		}
		for (int i = 0; i < playerRoots.size(); i++) {
			try {
				((Root) playerRoots.elementAt(i)).addOccupied(position);
				accepted = true;
				extended = (Root) playerRoots.elementAt(i);
			} catch (InvalidMoveException ime) {
				//Ignore
			}
		}
		if (!accepted)
			throw new InvalidMoveException("O ultimo lance nao pode ser adicionado a nenhuma raiz");
		positions[position.getX()][position.getY()] = turn;
		BoardPosition[] neighbours = null;
		Root r = null;
		for (int i = 0; i < playerRoots.size(); i++) {
			r = (Root) playerRoots.elementAt(i);
			neighbours = r.getNeighbourhood();
			r.removeAllFree();
			r.addFree(getValids(neighbours, turn));
		}
		for (int i = 0; i < adversaryRoots.size(); i++) {
			r = (Root) adversaryRoots.elementAt(i);
			r.removeFree(position);
		}
		if (extended.isDead()) {
			free(extended.occupied());
			playerRoots.remove(extended);
			removed = true;
		} else {
			for (int i = 0; i < playerRoots.size(); i++) {
				r = (Root) playerRoots.elementAt(i);
				if (r.isDead()) {
					free(r.occupied());
					playerRoots.remove(r);
					removed = true;
				}
			}
			for (int i = 0; i < adversaryRoots.size(); i++) {
				r = (Root) adversaryRoots.elementAt(i);
				if (r.isDead()) {
					free(r.occupied());
					adversaryRoots.remove(r);
					removed = true;
				}
			}
		}
		if (removed)
			renewAllRoots();
	}

    /*
     *  Versao especial de Mark para inicializacao do tabuleiro: apenas
     *  marca a posicao, nao verifica nada.
     */
	public void initialMark(BoardPosition position, int turn) {
		positions[position.getX()][position.getY()] = turn;
	}

    /*
     *  Determina o vencedor no momento. O valor retornado equivale a um
     *  dos atributos estaticos BLACK, WHITE e NOBODY desta mesma 
     *  classe.
     */
	public int winner() {
		if (whiteRoots.isEmpty()) {
			return BLACK;
		} else if (blackRoots.isEmpty()) {
			return WHITE;
		} else {
			return NOBODY;
		}
	}

    /*
     * Retorna TRUE se a posicao em questao e valida para o jogador 
     * representado por "turn".
     */
	protected boolean isValid(BoardPosition pos, int turn) {
		BoardPosition[] neighbours = pos.neighbours();
		int sameColour = 0;
		for (int i = 0; i < neighbours.length; i++) {
			if (positions[neighbours[i].getX()][neighbours[i].getY()] == turn)
				sameColour++;
		}
		System.out.println();
		return (sameColour == 1);
	}

    /*
     * Inicializa as raizes, isto e, prepara as raizes para o comeco do jogo.
     */
	protected void initializeRoots() throws InvalidConfiguration {
		try {
			if (dimension == 9) {
				// Initialize the black roots
				blackRoots.add(new Root(new BoardPosition("B2", dimension)));
				initialMark(new BoardPosition("B2", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("H8", dimension)));
				initialMark(new BoardPosition("H8", dimension), BLACK);
				// Initialize the white roots
				whiteRoots.add(new Root(new BoardPosition("H2", dimension)));
				initialMark(new BoardPosition("H2", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("B8", dimension)));
				initialMark(new BoardPosition("B8", dimension), WHITE);
			} else if (dimension == 19) {
				// Initialize the black roots
				blackRoots.add(new Root(new BoardPosition("A7", dimension)));
				initialMark(new BoardPosition("A7", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("A19", dimension)));
				initialMark(new BoardPosition("A19", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("G1", dimension)));
				initialMark(new BoardPosition("G1", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("G13", dimension)));
				initialMark(new BoardPosition("G13", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("N7", dimension)));
				initialMark(new BoardPosition("N7", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("N19", dimension)));
				initialMark(new BoardPosition("N19", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("T1", dimension)));
				initialMark(new BoardPosition("T1", dimension), BLACK);
				blackRoots.add(new Root(new BoardPosition("T13", dimension)));
				initialMark(new BoardPosition("T13", dimension), BLACK);
				// Initialize the white roots
				whiteRoots.add(new Root(new BoardPosition("A1", dimension)));
				initialMark(new BoardPosition("A1", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("A13", dimension)));
				initialMark(new BoardPosition("A13", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("G7", dimension)));
				initialMark(new BoardPosition("G7", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("G19", dimension)));
				initialMark(new BoardPosition("G19", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("N1", dimension)));
				initialMark(new BoardPosition("N1", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("N13", dimension)));
				initialMark(new BoardPosition("N13", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("T7", dimension)));
				initialMark(new BoardPosition("T7", dimension), WHITE);
				whiteRoots.add(new Root(new BoardPosition("T19", dimension)));
				initialMark(new BoardPosition("T19", dimension), WHITE);
			} else {
				throw new InvalidConfiguration(
					"Dimensao de tabuleiro invalida: " + dimension + ".");
			}
		} catch (InvalidMoveException ime) {
			//Ignore
		}
	}

	/*
	 * Recebe uma lista de posicoes e retorna uma lista contendo somente as posicoes livres para o jogador "turn".
	 */
	protected BoardPosition[] getValids(BoardPosition[] p, int turn) {
		Vector valid = new Vector();
		BoardPosition[] neighbours = null;
		int sameColour = 0;
		for (int i = 0; i < p.length; i++) {
			neighbours = p[i].neighbours();
			sameColour = 0;
			if (!occupied(p[i])) {
				for (int j = 0; j < neighbours.length; j++) {
					if (positions[neighbours[j].getX()][neighbours[j].getY()]
						== turn)
						sameColour++;
				}
				if (sameColour == 1)
					valid.add(p[i]);
			}
		}
		BoardPosition[] toReturn = new BoardPosition[valid.size()];
		for (int i = 0; i < toReturn.length; i++) {
			toReturn[i] = (BoardPosition) valid.elementAt(i);
		}
		return toReturn;
	}

	/*
	 * Libera as posicoes contidas em pos
	 */
	protected void free(BoardPosition[] pos) {
		for (int i = 0; i < pos.length; i++) {
			positions[pos[i].getX()][pos[i].getY()] = FREE;
		}
	}

	/*
	 * Rorganiza todas as posicoes livres de todas as raizes
	 */
	protected void renewAllRoots() {
		Root aux = null;
		for (int i = 0; i < blackRoots.size(); i++) {
			aux = (Root) blackRoots.elementAt(i);
			aux.renewFree(getValids(aux.getNeighbourhood(), BLACK));
		}
		for (int i = 0; i < whiteRoots.size(); i++) {
			aux = (Root) whiteRoots.elementAt(i);
			aux.renewFree(getValids(aux.getNeighbourhood(), WHITE));
		}
	}

}
