package mc600.gamemanager;

import mc600.boardgames.*;
import java.io.*;

public class Judge {
	// p[0] not used; p[1] and p[2] are the opponents
	private PlayerClient p[] = new PlayerClient[3];
	private mc600.boardgames.TanboBoard board;
	/**
	 * Construtor do juiz.
	 *
	 * Este objeto repassa os comandos dos jogadores para o tabuleiro,
	 * controlando o timeout para cada lance, apos o termino do jogo,
	 * incrementa a contagem de vitorias do jogador vencedor.
	 *
	 * Creation date: (3/8/00 9:26:09 PM)
	 * @param p1 mc600.gamemanager.PlayerClient
	 * @param p2 mc600.gamemanager.PlayerClient
	 */
	/**
	 * Funcao principal do Juiz
	 *
	 *
	 * O objeto juiz e' encarregado de solicitar os lances aos jogadores,
	 * e repassa-los para	
	 * o tabuleiro, que decide sobre a validade dos mesmos.
	 *
	 * Existem 3 maneiras anormais de o jogo ser encerrado:
	 * 
	 *  1- O jogador efetua um lance invalido
	 *  2- O jogador demora mais que "Player.MAXRESPONSETIME" para jogar
	 *  3- O jogador provoca um erro de IO ao enviar o lance.
	 *
	 * Nos tres casos o infrator perde a partida.
	 * Sera gravado um log do juiz informando-lhe do
	 * ocorrido. :-)
	 *
	 * Se o jogo terminar normalmente, o vencedor e' determinado
	 * de acordo com as regras do jogo (ou pode haver empate).
	 *
	 */
	public Judge(PlayerClient p1, PlayerClient p2) throws IOException {

		p[1] = p1;
		p[2] = p2;

		// verifica se jogadores sao validos
		// (um deles pode ser jogador de paridade)
		if ((p[1] != null) && (p[2] != null)) {

			String resultado;

			String ini =
				"Partida <"
					+ p[1].getPlayerName()
					+ "> X <"
					+ p[2].getPlayerName()
					+ ">\n";
			p[1].log(ini);
			p[2].log(ini);

			int vez = 0;
			try {
				// "restarta" jogadores para nova partida
				// se chegou aqui e' porque os jogadores foram criados,
				// logo, se houver excecao significa que jogador
				// caiu por algum motivo.  Perde por WO.
				boolean pok[] = new boolean[3];

				for (int i = 1; i <= 2; i++)
					try {
						p[i].shutdown();
						pok[i] = p[i].restart();
					} catch (Exception e) {
						pok[i] = false;
					}

				if (pok[1] && pok[2]) {

					// inicializa tabuleiro
					board = new TanboBoard(Player.DIM);

					// faz jogo

					vez = 1;
					// flag indicando se jogador ganhou outro lance
					// (isso para jogos onde `as vezes algum jogador
					// joga de novo)
					boolean ganhouLance = false;

					try {

						// Primeiro lance deve ser comando para
						// jogador iniciar partida
						String move = Player.STARTCMD;
						int lance = 0;

						// continua enquanto houver linhas livres
						while (!board.gameOver()) {

							// manda lance para jogador "da vez",
							// recebendo replica
							move = p[vez].play(move);
							// registra
							lance++;
							String str =
								(lance % 2 == 1)
									? (lance + 1) / 2 + ". " + move + " , "
									: move + "\n";
							p[vez].log(str);
							p[3 - vez].log(str);
							// marca no tabuleiro,
							// se for o caso
							if (ganhouLance)
								ganhouLance = false;
							else
								ganhouLance =
									board.play(move, p[vez].equals(p1));

							// troca vez
							vez = 3 - vez;

						} // end while not gameOver

					} catch (InvalidMoveException e) {

						p[vez].incJogInvalida();
						p[3 - vez].incBoasPartidas();
						p[3 - vez].incVictoryCount();
						resultado =
							"P" + vez + ": jogada invalida: " + e.getMessage();

						// grava logs...
						p[vez].log(resultado + "\n");
						p[3 - vez].log(resultado + "\n");

					} catch (IOException e) {

						p[vez].incTempEsgotado();
						p[3 - vez].incBoasPartidas();
						p[3 - vez].incVictoryCount();
						resultado = "P" + vez + ": tempo esgotado";

						// grava logs...
						p[vez].log(resultado + "\n");
						p[3 - vez].log(resultado + "\n");

					} catch (Exception e) {

						p[vez].incOutrosErros();
						p[3 - vez].incBoasPartidas();
						p[3 - vez].incVictoryCount();
						resultado =
							"P" + vez + ": gerou excecao:" + e.toString();

						// grava logs...
						p[vez].log(resultado + "\n");
						p[3 - vez].log(resultado + "\n");
					}

					if (board.gameOver()) {
						// Fim de jogo normal, determina quem foi o vencedor
						// ambos fizeram boa partida
						p[1].incBoasPartidas();
						p[2].incBoasPartidas();
						if (board.victoryBlack()) {
							// p1 venceu
							p[1].incVictoryCount();
							resultado = "ganhou P1";
						} else if (board.victoryWhite()) {
							// p2 venceu
							p[2].incVictoryCount();
							resultado = "ganhou P2";
						} else {
							// empate (pode ocorrer jah que as dimensoes
							// do tabuleiro sao variaveis...
							p[1].incDrawCount();
							p[2].incDrawCount();
							resultado = "empate";
						}
						p[1].log(resultado + "\n");
						p[2].log(resultado + "\n");
					}

				} else {
					// algum dos dois nao ok; quem ok ganha
					// o outro nao incrementa no. de boas partidas
					resultado = "nenhum dos dois restartou";
					for (int i = 1; i <= 2; i++) {
						if (pok[i]) {
							p[i].incVictoryCount();
							p[i].incBoasPartidas();
							resultado = "P" + (2 - i) + " nao restartou";
						} else {
							p[i].incOutrosErros();
						}
					}
					p[1].log(resultado + "\n");
					p[2].log(resultado + "\n");
				}

				//--------------------------------------------------
				try {
					p1.shutdown();
					p2.shutdown();
				} catch (IOException e) {
					System.out.println(
						"Erro ao finalizar jogadores: " + e.getMessage());
				}

			} catch (Exception e) {
				System.out.println(
					p1.getPlayerName() + " x " + p2.getPlayerName());
				System.out.println("jogando: " + vez);
				System.out.println("Erro: " + e.getMessage());
			}
		}
	}
}
