package mc600.gamemanager;

import java.io.*;
import java.net.*;

public class PlayerClient extends Player {
    private Socket skt;
    private int victoryCount = 0;
    private int drawCount    = 0;
    private int boasPartidas = 0;
    private int jogInvalida  = 0;
    private int tempEsgotado = 0;
    private int outrosErros  = 0;
    private long timeLeft = MAXGAMETIME;
    private InetAddress addr = null;
    private int port = 0;

    private FileWriter log;

    /**
     * Tenta criar cliente do jogador no addr, porta port...
     *
     * @param port int
     * @param id java.lang.String
     * @exception java.lang.Exception The exception description.
     */
    public PlayerClient(InetAddress addr, int port) throws Exception {

	super("");

	this.port = port;
	this.addr = addr;

	// Tenta conectar, se a porta nao esta alocada, levanta excesao
	skt = new Socket (addr, port);

	//	skt.setTcpNoDelay(true);	


	// Valida jogador.  Se tudo correr bem,
	// nome do jogador sera registrado e o objeto criado
	setPlayerName(validatePlayer(skt));

    }

    public int getVictoryCount() {
	return victoryCount;
    }

    public int getDrawCount() {
	return drawCount;
    }

    public int getJogInvalida() {
	return jogInvalida;
    }

    public int getTempEsgotado() {
	return tempEsgotado;
    }

    public int getOutrosErros() {
	return outrosErros;
    }

    public int getBoasPartidas() {
	return boasPartidas;
    }

    private boolean hasTimetoPlay() {

	return (timeLeft > 0);
    }

    /**
     * Funcao recebe lance "move", passando para o server,
     * e aguardando resposta.  Bloqueante.
     *
     * Creation date: (3/22/00 11:55:51 PM)
     * @return java.lang.String
     * @param move java.lang.String
     * @exception java.lang.Exception The exception description.
     */
    public String play(String move) throws IOException {

	byte[] bmove = new byte[MOVESIZE];
	long elapsedTime=0;

	if (!hasTimetoPlay())

	    throw new IOException("Excedido tempo maximo de jogo");

	// captura hora atual
	elapsedTime =  System.currentTimeMillis();

	// ajusta o timeout do socket para derrubar jogador
	// caso demore muito...
	skt.setSoTimeout(Player.MAXRESPONSETIME);
	   	
	DataOutputStream dataOut =
	    new DataOutputStream (skt.getOutputStream ());
	DataInputStream dataIn =
	    new DataInputStream (skt.getInputStream ());

	// Envia ao servidor o lance
	dataOut.writeBytes(move);
	dataOut.flush ();
		
	// Recebe lance do servidor
	dataIn.readFully(bmove);

	// desliga o timeout
	skt.setSoTimeout(0);

	// calcula tempo decorrido
	elapsedTime = System.currentTimeMillis() - elapsedTime;
	timeLeft -= elapsedTime;		

	// Retorna resposta do servidor
	return (new String(bmove));
    }

    /**
     * Reinicia socket, verificando se o mesmo jogador continua na porta
     * inicialmente alocada.
     * Caso positivo, retorna true. caso contrario, false.
     *
     * Creation date: (3/25/00 2:07:20 AM)
     * @exception java.lang.Exception The exception description.
     */
    public boolean restart() throws Exception {

	String novoNome;

	timeLeft = Player.MAXGAMETIME;
	
	// Tenta conectar, se a porta nao esta alocada, levanta excesao
	skt = new Socket (addr, port);

	skt.setTcpNoDelay(true);
	
	novoNome = validatePlayer(skt);

	// Valida jogador, e verifica se nome nao mudou.
	return (novoNome.compareTo(getPlayerName()) == 0);
    }

    /**
     * Insert the method's description here.
     * Creation date: (3/24/00 12:12:44 AM)
     * @param newVictoryCount int
     */
    protected void incVictoryCount() {
	victoryCount++;
    }
 
    protected void incDrawCount() {
	drawCount++;
    }
 
    protected void incBoasPartidas() {
	boasPartidas++;
    }

    protected void incJogInvalida() {
	jogInvalida++;
    }

    protected void incTempEsgotado() {
	tempEsgotado++;
    }

    protected void incOutrosErros() {
	outrosErros++;
    }

    /**
     * Insert the method's description here.
     * Creation date: (3/23/00 12:07:50 AM)
     * @exception java.io.IOException The exception description.
     */
    public void shutdown() throws IOException {
	
	skt.close();

    }

    /**
     * Verifica se o servidor alocado em "skt" eh valido.
     * Se for, retorna nome, senao, levanta excessao.
     *
     * Creation date: (3/25/00 2:09:43 AM)
     * @return java.lang.String
     */
    private String validatePlayer(Socket skt) throws Exception{

	byte[] buffer = new byte[MAGIC.length()];
	String magic;
	
	// ajusta o timeout do socket para derrubar jogador
	// caso demore muito...
	skt.setSoTimeout(Player.MAXVALIDATETIME);
	
	DataInputStream dataIn =
	    new DataInputStream (skt.getInputStream ());

	// Verifica se servidor eh valido, lendo MAGIC
	dataIn.readFully(buffer,0,MAGIC.length());
	magic = new String(buffer);
	
	if (magic.compareTo(MAGIC) != 0)

	    throw new Exception("Servidor invalido.");

	// le o tamanho do nome do jogador
	int nameLen = dataIn.readInt();
	if ( (nameLen <= 0) || (nameLen > MAXPLAYERNAME) )

	    throw new Exception("Tamanho do nome do jogador invalido.");
	
	// le o nome do jogador
	buffer = new byte[nameLen];
	dataIn.readFully(buffer);
	
	// desliga o timeout
	skt.setSoTimeout(0);
	
	return (new String(buffer));
    }

    public void openLog() {
	String filename = getPlayerName()+".tmp";
	try {
	    log = new FileWriter (filename, true);
	} catch (IOException e) {
	    System.err.println("Could not open " + filename);
	}
    }

    public void log(String message) {
	try {
	    log.write(message);
	} catch (IOException e) {
	    System.err.println("Could not write to the logfile of " +
			       getPlayerName());
	}
    }

    public void closeLog() {
	try {
	    log.close();
	} catch (IOException e) {
	    System.err.println("Could not close the logfile of " +
			       getPlayerName());
	}
    }	    
}
